@@ -1,6 +1,6 @@
 /*
  *
- * Copyright (c) 2013, Laurens van der Maaten (Delft University of Technology)
+ * Copyright (c) 2014, Laurens van der Maaten (Delft University of Technology)
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +38,7 @@
 #include <queue>
 #include <limits>
 #include <cmath>
-
+#include <Rcpp.h>
 
 #ifndef VPTREE_H
 #define VPTREE_H
@@ -185,6 +185,7 @@ private:
     // Function that (recursively) fills the tree
     Node* buildFromPoints( int lower, int upper )
     {
+        Rcpp::RNGScope scope;
         if (upper == lower) {     // indicates that we're done here!
             return NULL;
         }
@@ -196,7 +197,7 @@ private:
         if (upper - lower > 1) {      // if we did not arrive at leaf yet
             
             // Choose an arbitrary point and move it to the start
-            int i = (int) ((double)rand() / RAND_MAX * (upper - lower - 1)) + lower;
+            int i = (int) ((double)R::runif(0,1) * (upper - lower - 1)) + lower;
             std::swap(_items[lower], _items[i]);
             
             // Partition around the median distance
